@@ -24,8 +24,8 @@ By Dan Porter, PhD
 Diamond
 2017
 
-Version 3.1.0
-Last updated: 10/06/20
+Version 3.2.2
+Last updated: 27/09/21
 
 Version History:
 27/07/17 1.0    Version History started.
@@ -44,6 +44,9 @@ Version History:
 27/05/20 3.0    Updated write_cif for magnetic moments - now writes simple mcif structures
 09/06/20 3.0.1  Updated code for changes to fc.gen_sym_mat
 10/06/20 3.1    Updated Symmetry to include time operators
+02/09/20 3.2.0  Added Cell.reflection_hkl and transmission_hkl, added __str__ methods
+22/10/20 3.2.1  Added Cell.moment, updated Cell.latt()
+27/09/21 3.2.2  Added Cell.orientation, updated Cell.UV()
 
 @author: DGPorter
 """
@@ -54,13 +57,14 @@ from warnings import warn
 # Internal functions
 from . import functions_general as fg, functions_crystallography as fc
 from . import functions_crystallography as fc
+from .classes_orientation import Orientation
 from .classes_properties import Properties
 #from .classes_orbitals import CrystalOrbitals
 from .classes_scattering import Scattering
 from .classes_multicrystal import MultiCrystal
 from .classes_plotting import Plotting, PlottingSuperstructure
 
-__version__ = '3.1.0'
+__version__ = '3.2.2'
 
 
 class Crystal:
@@ -142,7 +146,12 @@ class Crystal:
         :return: None
         """
 
-        uvw, type, label, occ, uiso, mxmymz = self.Atoms.get()
+        uvw, atom_type, label, occ, uiso, mxmymz = self.Atoms.get()
+
+        # Shortcut if only 1 symmetry operation
+        if len(self.Symmetry.symmetry_operations) == 1:
+            self.Structure = Atoms(uvw[:, 0], uvw[:, 1], uvw[:, 2], atom_type, label, occ, uiso, mxmymz)
+            return
 
         new_uvw = np.empty([0, 3])
         new_type = []
@@ -159,7 +168,7 @@ class Crystal:
             # Append to P1 atoms arrays
             new_uvw = np.append(new_uvw, sympos, axis=0)
             new_mxmymz = np.append(new_mxmymz, symmag, axis=0)
-            new_type += [type[n]] * Nsympos
+            new_type += [atom_type[n]] * Nsympos
             new_label += [label[n]] * Nsympos
             new_occupancy += [occ[n]] * Nsympos
             new_uiso += [uiso[n]] * Nsympos
@@ -208,19 +217,20 @@ class Crystal:
 
         if filename is None:
             filename = '%s' % fg.saveable(self.name)
+        cifvals['FileTitle'] = filename
 
         if self.Atoms.ismagnetic():
             fc.write_mcif(cifvals, filename, comments)
         else:
             fc.write_cif(cifvals, filename, comments)
 
-    def new_cell(self, latt=[1.0]):
+    def new_cell(self, lattice_parameters=(), *args, **kwargs):
         """
         Replace the lattice parameters
-        :param latt: [a,b,c,alpha,beta,gamma]
+        :param lattice_parameters: [a,b,c,alpha,beta,gamma]
         :return: None
         """
-        self.Cell.latt(latt)
+        self.Cell.latt(lattice_parameters, *args, **kwargs)
 
     def new_atoms(self, u=[0], v=[0], w=[0], type=None,
                   label=None, occupancy=None, uiso=None, mxmymz=None):
@@ -287,6 +297,7 @@ class Crystal:
         """
 
         super = Superstructure(self, P)
+        super.generate_super_positions()
         return super
 
     def add_parent(self, parent, P):
@@ -297,15 +308,13 @@ class Crystal:
             su = xtl.add_parent(parent, [[1,1,0],[0,2,0],[0,0,1]])
         """
 
-        uvw, type, label, occupancy, uiso, mxmymz = self.Structure.get()
+        uvw, atom_type, label, occupancy, uiso, mxmymz = self.Structure.get()
         latt = self.Cell.lp()
 
         super = Superstructure(parent, P)
         super.name = self.name
-
         super.new_cell(latt)
-        super.new_atoms(uvw[:, 0], uvw[:, 1], uvw[:, 2], type, label, occupancy, uiso, mxmymz)
-
+        super.new_atoms(uvw[:, 0], uvw[:, 1], uvw[:, 2], atom_type, label, occupancy, uiso, mxmymz)
         return super
 
     def start_gui(self):
@@ -346,6 +355,9 @@ class Crystal:
         fmt = "%s with %d atomic positions, %d symmetries"
         return fmt % (self.name, len(self.Atoms.type), len(self.Symmetry.symmetry_operations))
 
+    def __str__(self):
+        return self.info()
+
 
 class Cell:
     """
@@ -357,6 +369,7 @@ class Cell:
         UC.latt([2.85,2.85,10.8,90,90,120]) # Define the lattice parameters from a list
         UC.tth([0,0,12],energy_kev=8.0) # Calculate the two-theta of a reflection
         UC.lp() # Returns the current lattice parameters
+        UC.orientation # class to chanage cell orientation in space
     """
     _required_cif = [
         "_cell_length_a",
@@ -368,36 +381,32 @@ class Cell:
     ]
 
     def __init__(self, a=1.0, b=1.0, c=1.0, alpha=90., beta=90.0, gamma=90.0):
-        self.latt([a, b, c, alpha, beta, gamma])
+        self.a = float(a)
+        self.b = float(b)
+        self.c = float(c)
+        self.alpha = float(alpha)
+        self.beta = float(beta)
+        self.gamma = float(gamma)
+        self.orientation = Orientation()
 
-    def latt(self, lattice_parameters):
+    def latt(self, lattice_parameters=(), *args, **kwargs):
         """ 
         Generate lattice parameters with list
-          latt([1]) -> a=b=c=1,alpha=beta=gamma=90
+          latt(1) -> a=b=c=1,alpha=beta=gamma=90
           latt([1,2,3]) -> a=1,b=2,c=3,alpha=beta=gamma=90
           latt([1,2,3,120]) -> a=1,b=2,c=3,alpha=beta=90,gamma=120
           latt([1,2,3,10,20,30]) -> a=1,b=2,c=3,alpha=10,beta=20,gamma=30
+          latt(1,2,3,10,20,30) -> a=1,b=2,c=3,alpha=10,beta=20,gamma=30
+          latt(a=1,b=2,c=3,alpha=10,beta=20,gamma=30]) -> a=1,b=2,c=3,alpha=10,beta=20,gamma=30
         """
 
-        self.a = float(lattice_parameters[0])
-        self.alpha = 90.0
-        self.beta = 90.0
-        self.gamma = 90.0
-
-        if len(lattice_parameters) == 1:
-            self.b = 1.0 * self.a
-            self.c = 1.0 * self.a
-        else:
-            self.b = float(lattice_parameters[1])
-            self.c = float(lattice_parameters[2])
-
-        if len(lattice_parameters) == 4:
-            self.gamma = 120.0
-
-        if len(lattice_parameters) == 6:
-            self.alpha = float(lattice_parameters[3])
-            self.beta = float(lattice_parameters[4])
-            self.gamma = float(lattice_parameters[5])
+        lp = fc.gen_lattice_parameters(lattice_parameters, *args, **kwargs)
+        self.a = lp[0]
+        self.b = lp[1]
+        self.c = lp[2]
+        self.alpha = lp[3]
+        self.beta = lp[4]
+        self.gamma = lp[5]
 
     def fromcif(self, cifvals):
         """
@@ -423,7 +432,7 @@ class Cell:
         beta, dbeta = fg.readstfm(cifvals['_cell_angle_beta'])
         gamma, dgamma = fg.readstfm(cifvals['_cell_angle_gamma'])
 
-        self.latt([a, b, c, alpha, beta, gamma])
+        self.latt(a, b, c, alpha, beta, gamma)
 
     def update_cif(self, cifvals):
         """
@@ -454,7 +463,7 @@ class Cell:
          Returns the unit cell as a [3x3] array, [A,B,C]
          The vector A is directed along the x-axis
         """
-        return fc.latpar2UV_rot(*self.lp())
+        return self.orientation(fc.latpar2uv_rot(*self.lp()))
 
     def UVstar(self):
         """
@@ -473,6 +482,9 @@ class Cell:
     def Bmatrix(self):
         """
         Calculate the Busing and Levy B matrix from a real space UV
+         "choose the x-axis parallel to a*, the y-axis in the plane of a* and b*, and the z-axis perpendicular to
+         that plane"
+         W. R. Busing & H. A. Levy, Acta  Cryst.  (1967). 22,  457
         """
         return fc.Bmatrix(self.UV())
 
@@ -486,7 +498,10 @@ class Cell:
         return out
 
     def __repr__(self):
-        return 'a = %1.5g A,  b = %1.5f A,  c = %1.5g A,  A = %1.5g,  B = %1.5g,  G = %1.5g' % self.lp()
+        return 'Cell(a=%1.5g, b=%1.5f, c=%1.5g, alpha=%1.5g, beta=%1.5g, gamma=%1.5g)' % self.lp()
+
+    def __str__(self):
+        return self.info()
 
     def generate_lattice(self, U, V, W):
         """
@@ -546,6 +561,14 @@ class Cell:
         R = np.reshape(np.asarray(R, dtype=np.float), [-1, 3])
         return fc.indx(R, self.UV())
 
+    def moment(self, mxmymz):
+        """Calcualte moment from value stored in cif"""
+        momentmag = fg.mag(mxmymz).reshape([-1, 1])
+        momentxyz = self.calculateR(mxmymz)
+        mom = momentmag * fg.norm(momentxyz)  # broadcast n*1 x n*3 = n*3
+        mom[np.isnan(mom)] = 0.
+        return mom
+
     def Qmag(self, HKL):
         """
         Returns the magnitude of wave-vector transfer of [h,k,l], in A-1
@@ -601,13 +624,13 @@ class Cell:
             angle[n] = np.rad2deg(fg.ang(Q[n], parallel)) - 90 + theta_offset
         return (tth / 2) + angle
 
-    def dspace(self, HKL):
+    def dspace(self, hkl):
         """
         Calculate the d-spacing in A
-        :param HKL: array : list of reflections
+        :param hkl: array : list of reflections
         :return: d-spacing
         """
-        Qmag = self.Qmag(HKL)
+        Qmag = self.Qmag(hkl)
         return fc.q2dspace(Qmag)
 
     def max_hkl(self, energy_kev=8.048, max_angle=180.0):
@@ -648,26 +671,66 @@ class Cell:
         Qm = self.Qmag(HKL)
         return HKL[Qm <= Qmax, :]
 
-    def sort_hkl(self, HKL, ascend=True):
+    def reflection_hkl(self, energy_kev=8.048, max_angle=180.0,
+                       specular=(0, 0, 1), theta_offset=0, min_theta=0, max_theta=180.):
+        """
+        Returns an array of all (h,k,l) reflections in reflection geometry
+        :param energy_kev: energy in keV
+        :param max_angle: max two-theta angle
+        :param specular: (h,k,l) of direction normal to surface and the incident beam
+        :param theta_offset: float : angle (deg) of surface relative to specular normal
+        :param min_theta: float : cut hkl reflections with reflection-theta lower than min_theta
+        :param max_theta: flaot : cut hkl reflections with reflection-theta greater than max_theta
+        :return: array of hkl
+        """
+        hkl = self.all_hkl(energy_kev, max_angle)
+        tth = self.tth(hkl, energy_kev)
+        theta = self.theta_reflection(hkl, energy_kev, specular, theta_offset)
+
+        p1 = (theta > min_theta) * (theta < max_theta)
+        p2 = (tth > (theta + min_theta)) * (tth < (theta + max_theta))
+        return hkl[p1 * p2, :]
+
+    def transmission_hkl(self, energy_kev=8.048, max_angle=180.0,
+                         parallel=(0, 0, 1), theta_offset=0, min_theta=0, max_theta=180.):
+        """
+        Returns an array of all (h,k,l) reflections in reflection geometry
+        :param energy_kev: energy in keV
+        :param max_angle: max two-theta angle
+        :param parallel: (h,k,l) of direction normal to surface, parallel to the incident beam
+        :param theta_offset: float : angle (deg) of surface relative to specular normal
+        :param min_theta: float : cut hkl reflections with reflection-theta lower than min_theta
+        :param max_theta: flaot : cut hkl reflections with reflection-theta greater than max_theta
+        :return: array of hkl
+        """
+        hkl = self.all_hkl(energy_kev, max_angle)
+        tth = self.tth(hkl, energy_kev)
+        theta = self.theta_transmission(hkl, energy_kev, parallel, theta_offset)
+
+        p1 = (theta > min_theta) * (theta < max_theta)
+        p2 = (tth > (theta + min_theta)) * (tth < (theta + max_theta))
+        return hkl[p1 * p2, :]
+
+    def sort_hkl(self, hkl, ascend=True):
         """
         Returns array of (h,k,l) sorted by two-theta
-        :param HKL: array : list of [h,k,l] values
+        :param hkl: array : list of [h,k,l] values
         :param ascend: True*/False : if False, lowest two-theta
         :return: HKL[sorted,:]
         """
 
-        HKL = np.reshape(np.asarray(HKL, dtype=np.float), [-1, 3])
-        Qm = self.Qmag(HKL)
+        hkl = np.reshape(np.asarray(hkl, dtype=np.float), [-1, 3])
+        Qm = self.Qmag(hkl)
         idx = np.argsort(Qm)
-        return HKL[idx, :]
+        return hkl[idx, :]
 
-    def powder_average(self, HKL):
+    def powder_average(self, hkl):
         """
         Returns the powder average correction for the given hkl
-        :param HKL: array : list of reflections
+        :param hkl: array : list of reflections
         :return: correction
         """
-        q = self.Qmag(HKL)
+        q = self.Qmag(hkl)
         return 1/(q+0.001)**2
 
     def find_close_reflections(self, hkl, energy_kev, max_twotheta=2, max_angle=10):
@@ -723,7 +786,7 @@ class Cell:
         maxq = np.sqrt(q_max ** 2 + q_max ** 2)
         hmax, kmax, lmax = fc.maxHKL(maxq, self.UVstar())
         HKL = fc.genHKL([hmax, -hmax], [kmax, -kmax], [lmax, -lmax])
-        HKL = HKL + centre  # reflection about central reflection
+        HKL = np.ceil(HKL + centre)  # reflection about central reflection
         Q = self.calculateQ(HKL)
 
         # generate box in reciprocal space
@@ -731,10 +794,54 @@ class Cell:
 
         box_coord = fg.index_coordinates(Q - c_cart, CELL)
         incell = np.all(np.abs(box_coord) <= 0.5, axis=1)
+        if np.sum(incell) < 1:
+            return np.zeros((0,)), np.zeros((0,)), np.zeros((0, 3))
         plane_coord = 2 * q_max * box_coord[incell, :]
         return plane_coord[:, 0], plane_coord[:, 1], HKL[incell, :]
 
 
+class Atom:
+    """
+    Atom class
+    Contains site information
+    """
+    def __init__(self, u, v, w, atom_type, label, occupancy=1.0, uiso=0.001, mxmymz=None):
+        self.u = u
+        self.v = v
+        self.w = w
+        self.type = atom_type
+        self.label = label
+        self.occupancy = occupancy
+        self.uiso = uiso
+        if mxmymz is None:
+            mxmymz = np.array([0., 0, 0])
+        self.mxmymz = mxmymz
+
+        props = fc.atom_properties(atom_type)  # returns a numpy structured array
+        prop_names = props.dtype.names
+        self.properties = props
+        for key in prop_names:
+            setattr(self, key, props[key][0])
+
+    def __repr__(self):
+        out = "Atom(%.3g, %.3g, %.3g, %s, %s)" % (self.u, self.v, self.w, self.type, self.label)
+        return out
+
+    def __str__(self):
+        out = "%s, %s, u = %6.3f, v = %6.3f, w = %6.3f, occ = %4.2f, uiso = %5.3f, mxmymz = (%.3g,%.3g,%.3g)"
+        mx, my, mz = self.mxmymz
+        return out % (self.label, self.type, self.u, self.v, self.w, self.occupancy, self.uiso, mx, my, mz)
+
+    def info(self):
+        """Display atomic properties"""
+
+        prop_names = self.properties.dtype.names
+        out = ''
+        for key in prop_names:
+            out += '%20s : %s\n' % (key, self.properties[key][0])
+        return out
+
+
 class Atoms:
     """
     Contains properties of atoms within the crystal
@@ -770,22 +877,22 @@ class Atoms:
         if type is None:
             self.type = np.asarray([self._default_atom] * Natoms)
         else:
-            self.type = type
+            self.type = np.asarray(type, dtype=str).reshape(-1)
         # label
         if label is None:
-            self.label = np.asarray([self._default_atom] * Natoms)
+            self.label = self.type.copy()
         else:
-            self.label = label
+            self.label = np.asarray(label, dtype=str).reshape(-1)
         # occupancy
         if occupancy is None:
             self.occupancy = np.ones(Natoms)
         else:
-            self.occupancy = occupancy
+            self.occupancy = np.asarray(occupancy, dtype=np.float).reshape(-1)
         # Uiso
         if uiso is None:
             self.uiso = self._default_uiso * np.ones(Natoms)
         else:
-            self.uiso = uiso
+            self.uiso = np.asarray(uiso, dtype=np.float).reshape(-1)
         # Mag vector mxmymz
         if mxmymz is None:
             self.mx = np.zeros(Natoms)
@@ -802,6 +909,9 @@ class Atoms:
         """Re-initialises the class, generating new atomic positions"""
         self.__init__(u, v, w, type, label, occupancy, uiso, mxmymz=None)
 
+    def __getitem__(self, idx):
+        return self.atom(idx)
+
     def fromcif(self, cifvals):
         """
         Import atom parameters from a cif dictionary
@@ -930,7 +1040,16 @@ class Atoms:
         #cifvals['_atom_site_moment.symmform'] = mag_sym
         return cifvals
 
-    def changeatom(self, idx, u=None, v=None, w=None, type=None,
+    def atom(self, idx):
+        """Create Atom object for atom site"""
+        idx = np.asarray(idx, dtype=int).reshape(-1)
+        atoms = [Atom(self.u[i], self.v[i], self.w[i], self.type[i], self.label[i],
+                      self.occupancy[i], self.uiso[i], self.mxmymz()[i]) for i in idx]
+        if len(atoms) == 1:
+            return atoms[0]
+        return atoms
+
+    def changeatom(self, idx=None, u=None, v=None, w=None, type=None,
                    label=None, occupancy=None, uiso=None, mxmymz=None):
         """
         Change an atoms properties
@@ -959,7 +1078,9 @@ class Atoms:
             self.type[idx] = type
 
         if label is not None:
-            self.label[idx] = label
+            old_labels = list(self.label)
+            old_labels[idx] = label
+            self.label = np.array(old_labels)
 
         if occupancy is not None:
             self.occupancy[idx] = occupancy
@@ -973,6 +1094,40 @@ class Atoms:
             self.my[idx] = mpos[:, 1]
             self.mz[idx] = mpos[:, 2]
 
+    def findatom(self, u=None, v=None, w=None, type=None,
+                 label=None, occupancy=None, uiso=None, mxmymz=None, tol=0.01):
+        """
+        Find atom using parameters, return idx
+        :param u: float
+        :param v: float
+        :param w: float
+        :param type: str
+        :param label: str
+        :param occupancy: float
+        :param uiso: float
+        :param mxmymz: [mx,my,mz]
+        :param tol: float, tolerance to match value
+        :return: array of indexes
+        """
+        all = np.ones(len(self.u))
+        if u is not None:
+            all *= np.abs(self.u - u) < tol
+        if v is not None:
+            all *= np.abs(self.v - v) < tol
+        if w is not None:
+            all *= np.abs(self.w - w) < tol
+        if type is not None:
+            all *= np.array(self.type) == type
+        if label is not None:
+            all *= np.array([True if label.lower() in a.lower() else False for a in self.label])
+        if occupancy is not None:
+            all *= np.abs(self.occupancy - occupancy) < tol
+        if uiso is not None:
+            all *= np.abs(self.uiso - uiso) < tol
+        if mxmymz is not None:
+            all *= fg.mag(self.mxmymz() - mxmymz) < tol
+        return np.where(all)[0]
+
     def addatom(self, u=0, v=0, w=0, type=None, label=None, occupancy=None, uiso=None, mxmymz=None):
         """
         Adds a new atom
@@ -1235,6 +1390,9 @@ class Atoms:
     def __repr__(self):
         return "%d sites, elements: %s" % (len(self.type), np.unique(self.type))
 
+    def __str__(self):
+        return self.info()
+
 
 class Symmetry:
     """
@@ -1299,16 +1457,19 @@ class Symmetry:
         self.spacegroup = spacegroup
 
         if '_symmetry_Int_Tables_number' in keys:
-            sgn = float(cifvals['_symmetry_Int_Tables_number'])
+            sgn = cifvals['_symmetry_Int_Tables_number']
         elif '_space_group_IT_number' in keys:
-            sgn = float(cifvals['_space_group_IT_number'])
+            sgn = cifvals['_space_group_IT_number']
         elif '_space_group_magn_number_BNS' in keys:
-            sgn = float(cifvals['_space_group_magn_number_BNS'].strip('\'"'))
+            sgn = cifvals['_space_group_magn_number_BNS'].strip('\'"')
         elif spacegroup == 'P1':
-            sgn = 1
+            sgn = '1'
         else:
-            sgn = 0
-        self.spacegroup_number = sgn
+            sgn = '0'
+        try:
+            self.spacegroup_number = float(sgn)
+        except ValueError:
+            self.spacegroup_number = 0
 
         self.generate_matrices()
 
@@ -1810,6 +1971,38 @@ class Symmetry:
             return multiplyers[0]
         return multiplyers
 
+    def spacegroup_name(self):
+        """Return the spacegroup name and number as str"""
+        spg = self.spacegroup.replace(' ', '')
+        spn = self.spacegroup_number
+        return "{} ({})".format(spg, spn)
+
+    def parity_time_info(self):
+        """
+        Returns string of parity and time operations for symmetry operations
+        :return: str
+        """
+        # From functions_crystallography.symmetry_ops2magnetic
+        operations = self.symmetry_operations
+        magnetic_ops = self.symmetry_operations_magnetic
+        # Convert operations to matrices
+        mat_ops = fc.gen_sym_mat(operations)
+        tim_ops = fc.sym_op_time(operations)
+
+        out = 'Spacegoup: %s (%s)\n' % (self.spacegroup, self.spacegroup_number)
+        out += '  n, Symmetry operations,   Time,    Parity,    T*P*M,  Magnetic Operation\n'
+        for n, mat in enumerate(mat_ops):
+            # Get time operation
+            t = tim_ops[n]
+            # Only use rotational part
+            m = mat[:3, :3]
+            # Get parity
+            p = np.linalg.det(m)
+            # Generate string
+            mag_str = fc.sym_mat2str(t * p * m)
+            out += "%3d, %19s, %6s, %9s, %8s, %19s\n" % (n, operations[n], t, p, mag_str, magnetic_ops[n])
+        return out
+
     def info(self):
         """
         Prints the symmetry information
@@ -1832,7 +2025,10 @@ class Symmetry:
         return out
 
     def __repr__(self):
-        return "%d symmetry operations" % len(self.symmetry_operations)
+        return "%s, %d symmetry operations" % (self.spacegroup_name(), len(self.symmetry_operations))
+
+    def __str__(self):
+        return self.info()
 
 
 class Superstructure(Crystal):
@@ -1873,22 +2069,32 @@ class Superstructure(Crystal):
         self.Cell = Cell()
         self.Symmetry = Symmetry()
         self.Atoms = Atoms()
-
-        # Add exta functions
-        # self.Plot = Plotting(self)
-        # self.Scatter = Scattering(self)
+        self.Structure = Atoms()
 
         self.name = Parent.name + ' supercell'
         self.P = np.asarray(P, dtype=np.float)
         self.Parent = Parent
         newUV = Parent.Cell.calculateR(P)
         self.new_cell(fc.UV2latpar(newUV))
+        self.scale = Parent.scale * np.prod(self.P)
+
+        # Add exta functions
+        self.Plot = PlottingSuperstructure(self)
+        self.Scatter = Scattering(self)
+        self.Properties = Properties(self)
+
+    def generate_super_positions(self):
+        """
+        Generate the supercell and superstructure based on P and parent structure
+        :return: None, set new atom positions
+        """
 
         # Build lattice of points
-        UVW = fc.genHKL(2 * np.max(P))
-        real_lattice = Parent.Cell.calculateR(UVW)
+        UVW = fc.genHKL(2 * np.max(self.P))
+        real_lattice = self.Parent.Cell.calculateR(UVW)
 
         # Determine lattice points within the cell
+        newUV = self.Parent.Cell.calculateR(self.P)
         indx_lattice = fc.indx(real_lattice, newUV)
         isincell = np.all(indx_lattice <= 0.99, axis=1) * np.all(indx_lattice > -0.01, axis=1)
         # for n in range(len(real_lattice)):
@@ -1898,21 +2104,21 @@ class Superstructure(Crystal):
         Ncell = len(UVW)
 
         # Increase scale to match size of cell
-        self.scale = Parent.scale * Ncell
+        self.scale = self.Parent.scale * Ncell
 
         # Get Parent atoms
-        uvw, type, label, occ, uiso, mxmymz = Parent.Structure.get()
+        uvw, atom_type, label, occ, uiso, mxmymz = self.Parent.Structure.get()
 
         # Generate all atomic positions
         new_uvw = np.empty([0, 3])
         new_mxmymz = np.empty([0, 3])
-        new_type = np.tile(type, Ncell)
+        new_type = np.tile(atom_type, Ncell)
         new_label = np.tile(label, Ncell)
         new_occ = np.tile(occ, Ncell)
         new_uiso = np.tile(uiso, Ncell)
-        for n in range(len(UVW)):
+        for n in range(Ncell):
             latpos = np.array([UVW[n, 0] + uvw[:, 0], UVW[n, 1] + uvw[:, 1], UVW[n, 2] + uvw[:, 2]]).T
-            latR = Parent.Cell.calculateR(latpos)
+            latR = self.Parent.Cell.calculateR(latpos)
             latuvw = fc.indx(latR, newUV)
 
             # Append to P1 atoms arrays
@@ -1923,11 +2129,6 @@ class Superstructure(Crystal):
                        type=new_type, label=new_label, occupancy=new_occ, uiso=new_uiso,
                        mxmymz=new_mxmymz)
 
-        # Add exta functions
-        self.Plot = PlottingSuperstructure(self)
-        self.Scatter = Scattering(self)
-        self.Properties = Properties(self)
-
     def set_scale(self):
         """
         Set scale parameter automatically
@@ -1997,4 +2198,4 @@ class Superstructure(Crystal):
 
 
 if __name__ == '__main__':
-    xtl = Crystal()
\ No newline at end of file
+    xtl = Crystal()
