@@ -7,8 +7,8 @@ By Dan Porter, PhD
 Diamond
 2017
 
-Version 1.5
-Last updated: 27/05/20
+Version 1.7
+Last updated: 26/01/21
 
 Version History:
 10/11/17 0.1    Program created
@@ -18,6 +18,8 @@ Version History:
 15/08/19 1.3    Added molcharge
 30/03/20 1.4    Added latex_table, info returns str, removed getattr from xray_edges, check if element exists
 12/05/20 1.5    Added orbitals function, exchange_paths
+15/10/20 1.6    Added scattering lengths + factors
+26/01/21 1.7    Added calculation of xray attenuation, transmission and refractive index
 
 
 @author: DGPorter
@@ -29,12 +31,18 @@ from . import functions_general as fg
 from . import functions_crystallography as fc
 from .classes_orbitals import CrystalOrbitals
 
-__version__ = '1.5'
+__version__ = '1.7'
 
 
 class Properties:
     """
     Properties functions for the Crystal Object
+    Example Usage:
+        xtl = Crystal('file.cif')
+        print(xtl.Properties)
+        xtl.Properties.density()
+        print(xtl.Properties.molcharge())
+        xtl.Properties.Co.K  # returns K edge of element Co (if Co in Crystal)
     """
     def __init__(self, xtl):
         """initialise"""
@@ -78,6 +86,31 @@ class Properties:
         """Return the molecular weight in g/mol"""
         return self.xtl.Structure.weight()
 
+    def neutron_scatteringlengths(self):
+        """
+        Return the bound coherent neutron scattering length for each atom in the structure
+        :return: [m] array of scattering lengths for each atom
+        """
+        return fc.neutron_scattering_length(self.xtl.Structure.type)
+
+    def xray_scattering_factor(self, hkl):
+        """
+        Return the x-ray scattering factor for given hkl reflections
+        :param hkl: [nx3] array of (h,k,l) reflections
+        :return: [nxm] array of scattering factors for each atom and reflection
+        """
+        qmag = self.xtl.Cell.Qmag(hkl)
+        return fc.xray_scattering_factor(self.xtl.Structure.type, qmag)
+
+    def magnetic_form_factor(self, hkl):
+        """
+        Return the magnetic form factor for given hkl reflections
+        :param hkl: [nx3] array of (h,k,l) reflections
+        :return: [nxm] array of scattering factors for each atom and reflection
+        """
+        qmag = self.xtl.Cell.Qmag(hkl)
+        return fc.magnetic_form_factor(self.xtl.Structure.type, qmag)
+
     def xray_edges(self):
         """
         Returns the x-ray absorption edges available and their energies
@@ -106,15 +139,15 @@ class Properties:
         :return: str
         """
 
-        type = self.xtl.Structure.type
+        atom_type = self.xtl.Structure.type
         occ = self.xtl.Structure.occupancy
 
         # Count elements
-        ats = np.unique(type)
+        ats = np.unique(atom_type)
         weights = fc.atom_properties(ats,'Weight')
         atno = np.zeros(len(ats))
         for n,element in enumerate(ats):
-            atno[n] = sum([occ[m] for m,x in enumerate(type) if x == element])
+            atno[n] = sum([occ[m] for m,x in enumerate(atom_type) if x == element])
 
         outstr=''
 
@@ -135,27 +168,27 @@ class Properties:
         :return: str
         """
 
-        type = np.asarray(self.xtl.Structure.type)
+        atom_type = np.asarray(self.xtl.Structure.type)
         occ = np.asarray(self.xtl.Structure.occupancy)
 
         # Count elements
-        ats = np.unique(type)
+        ats = np.unique(atom_type)
         ats = fc.arrange_atom_order(ats)  # arrange this by alcali/TM/O
         atno = {}
         for a in ats:
-            atno[a] = sum([occ[n] for n,x in enumerate(type) if x == a])
+            atno[a] = sum([occ[n] for n,x in enumerate(atom_type) if x == a])
 
         # default - set max Z atom to it's occupancy
         if element is None:
             z = fc.atom_properties(ats, 'Z')
-            element = ats[ np.argmax(z) ]
+            element = ats[np.argmax(z)]
             #element = min(atno, key=atno.get)
 
         if element_no is None:
-            element_no = np.max(occ[type == element])
+            element_no = np.max(occ[atom_type == element])
 
         divideby = atno[element]/ element_no
-        name = fc.count_atoms(type, occ, divideby, latex)
+        name = fc.count_atoms(atom_type, occ, divideby, latex)
         name = ''.join(name)
         return name
 
@@ -169,27 +202,27 @@ class Properties:
         :return: str
         """
 
-        type = np.asarray(self.xtl.Structure.type)
+        atom_type = np.asarray(self.xtl.Structure.type)
         occ = np.asarray(self.xtl.Structure.occupancy)
 
         # Count elements
-        ats = np.unique(type)
+        ats = np.unique(atom_type)
         ats = fc.arrange_atom_order(ats)  # arrange this by alcali/TM/O
         atno = {}
         for a in ats:
-            atno[a] = sum([occ[n] for n,x in enumerate(type) if x == a])
+            atno[a] = sum([occ[n] for n,x in enumerate(atom_type) if x == a])
 
         # default - set max Z atom to it's occupancy
         if element is None:
             z = fc.atom_properties(ats, 'Z')
-            element = ats[ np.argmax(z) ]
+            element = ats[np.argmax(z)]
             #element = min(atno, key=atno.get)
 
         if element_no is None:
-            element_no = np.max(occ[type == element])
+            element_no = np.max(occ[atom_type == element])
 
         divideby = atno[element]/ element_no
-        name = fc.count_charges(type, occ, divideby, latex)
+        name = fc.count_charges(atom_type, occ, divideby, latex)
         name = ' '.join(name)
         return name
 
@@ -203,14 +236,81 @@ class Properties:
 
         # determine u/p = sum(wi*(u/p)i) http://physics.nist.gov/PhysRefData/XrayMassCoef/chap2.html
         wi = self.xtl.Structure.mass_fraction()
-        Z = fc.atom_properties(self.xtl.Structure.type ,'Z')
+        Z = fc.atom_properties(self.xtl.Structure.type, 'Z')
         upi = fc.attenuation(Z, energy_kev)
         up = np.sum(wi*upi, axis=1)
 
         u = up*self.density()/10000
         return u
 
-    def diamagnetic_susceptibility(self, type='volume'):
+    def xray_transmission(self, energy_kev=None, thickness_um=100):
+        """
+        Calculate transmission of x-ray through a slab of material
+        Equivalent to https://henke.lbl.gov/optical_constants/filter2.html
+        Based on formulas from: Henke, Gullikson, and Davis, Atomic Data and Nuclear Data Tables 54 no.2, 181-342 (July 1993)
+        :param energy_kev: float or array, x-ray energy in keV
+        :param thickness_um: slab thickness in microns
+        :return: float or array
+        """
+        if energy_kev is None:
+            energy_kev = fc.getenergy()
+
+        atom_type = self.xtl.Structure.type
+        occ = self.xtl.Structure.occupancy
+        natoms = np.sum(occ)
+        vol = self.volume()
+        atom_per_volume = natoms / vol # atoms per A^3
+        elements = ['%s%s' % (at, o) for at, o in zip(atom_type, occ)]
+        return fc.xray_transmission(elements, energy_kev, atom_per_volume, thickness_um)
+
+    def xray_attenuation_length(self, energy_kev=None, grazing_angle=90):
+        """
+        Calcualte X-Ray Attenuation Length
+        Equivalent to: https://henke.lbl.gov/optical_constants/atten2.html
+        Based on formulas from: Henke, Gullikson, and Davis, Atomic Data and Nuclear Data Tables 54 no.2, 181-342 (July 1993)
+        :param energy_kev: float or array, x-ray energy in keV
+        :param grazing_angle: incidence angle relative to the surface, in degrees
+        :return: float or array, in microns
+        """
+        if energy_kev is None:
+            energy_kev = fc.getenergy()
+
+        atom_type = self.xtl.Structure.type
+        occ = self.xtl.Structure.occupancy
+        natoms = np.sum(occ)
+        vol = self.volume()
+        atom_per_volume = natoms / vol  # atoms per A^3
+        elements = ['%s%s' % (at, o) for at, o in zip(atom_type, occ)]
+        return fc.xray_attenuation_length(elements, energy_kev, atom_per_volume, grazing_angle)
+
+    def xray_reflectivity(self, energy_kev=None, grazing_angle=2):
+        """
+        Calculate the specular reflectivity of a material
+          NOT CURRENTLY WORKING
+        :param elements: str or list of str, if list - absorption will be summed over elements
+        :param energy_kev: float array
+        :param grazing_angle: incidence angle relative to the surface, in degrees
+        :return: float or array
+        """
+        if energy_kev is None:
+            energy_kev = fc.getenergy()
+        raise Warning("Warning: this doesn\'t work yet")
+
+        atom_type = self.xtl.Structure.type
+        occ = self.xtl.Structure.occupancy
+        natoms = np.sum(occ)
+        vol = self.volume()
+        atom_per_volume = natoms / vol  # atoms per A^3
+        elements = ['%s%s' % (at, o) for at, o in zip(atom_type, occ)]
+
+        refindex = fc.xray_refractive_index(elements, energy_kev, atom_per_volume)
+        costh = np.cos(np.deg2rad(grazing_angle))
+        ki = costh
+        kt = np.sqrt(refindex * np.conj(refindex) - costh ** 2)
+        r = (ki - kt) / (ki + kt)
+        return np.real(r * np.conj(r))
+
+    def diamagnetic_susceptibility(self, atom_type='volume'):
         """
         Calculate diamagnetic contribution to susceptibility
 
@@ -227,32 +327,32 @@ class Properties:
         # X = diamagnetic susceptibility, N=no ions, V = volume
         # Zeff = no valence electrons, r = ionic radius
 
-        C = (fg.u0*fg.e**2)/(6*fg.me)
-        V = self.xtl.Cell.volume()*1e-30 # m^3
-        M = self.xtl.Structure.weight()/fg.Na # g
-        mol = 1/fg.Na
+        C = (fg.u0 * fg.e ** 2) / (6 * fg.me)
+        V = self.xtl.Cell.volume() * 1e-30  # m^3
+        M = self.xtl.Structure.weight() / fg.Na  # g
+        mol = 1 / fg.Na
 
         # susceptibility type
-        if type.lower() == 'mass':
-            D = M/1000 # kg
-        elif type.lower() == 'cgs mass':
-            D = M*(4*fg.pi)
-        elif type.lower() == 'molar':
+        if atom_type.lower() == 'mass':
+            D = M / 1000  # kg
+        elif atom_type.lower() == 'cgs mass':
+            D = M * (4 * fg.pi)
+        elif atom_type.lower() == 'molar':
             D = mol
-        elif type.lower() == 'cgs molar':
-            D = mol*(4*fg.pi*1e-6)
-        elif type.lower() == 'cgs volume':
-            D = V*(4*fg.pi)
+        elif atom_type.lower() == 'cgs molar':
+            D = mol * (4 * fg.pi * 1e-6)
+        elif atom_type.lower() == 'cgs volume':
+            D = V * (4 * fg.pi)
         else:
             D = V
 
         atom_types = np.unique(self.xtl.Structure.type)
         X = 0.0
         for at in atom_types:
-            N = np.sum(self.xtl.Structure.occupancy[self.xtl.Structure.type==at])
-            Zeff = fc.atom_properties(str(at),'ValenceE')[0]
-            r = fc.atom_properties(str(at),'Radii')[0]*1e-12
-            X += -(N/D)*C*(Zeff*r*r)
+            N = np.sum(self.xtl.Structure.occupancy[self.xtl.Structure.type == at])
+            Zeff = fc.atom_properties(str(at), 'ValenceE')[0]
+            r = fc.atom_properties(str(at), 'Radii')[0] * 1e-12
+            X += -(N / D) * C * (Zeff * r * r)
         return X
 
     def atomic_neighbours(self, structure_index=0, radius=2.5, disp=False):
@@ -268,7 +368,7 @@ class Properties:
 
         Rindex = self.xtl.Cell.calculateR(self.xtl.Structure.uvw()[structure_index, :])
 
-        uvw, type, label, occ, uiso, mxmymz = self.xtl.Structure.generate_lattice(1, 1, 1)
+        uvw, atom_type, label, occ, uiso, mxmymz = self.xtl.Structure.generate_lattice(1, 1, 1)
 
         R = self.xtl.Cell.calculateR(uvw)
 
@@ -326,13 +426,13 @@ class Properties:
             print('Centre: %3d %4s (%6.2f,%6.2f,%6.2f)' % (cen_idx, cen_type, cen_uvw[0], cen_uvw[1], cen_uvw[2]))
 
         # Generate lattice of muliple cells to remove cell boundary problem
-        uvw, type, label, occ, uiso, mxmymz = self.xtl.Structure.generate_lattice(1, 1, 1)
+        uvw, atom_type, label, occ, uiso, mxmymz = self.xtl.Structure.generate_lattice(1, 1, 1)
         xyz = self.xtl.Cell.calculateR(uvw)
         all_bonds = cen_xyz - xyz
 
         # Exchange type
         # exchange_type = 1  # all atoms
-        exchange_allowed = type == exchange_type
+        exchange_allowed = atom_type == exchange_type
 
         # Inside unit cell
         if search_in_cell:
@@ -342,7 +442,7 @@ class Properties:
 
         # Bond distances
         mag = fg.mag(all_bonds)
-        neighbor_idx = np.where(incell * (type == cen_type) * (mag < nearest_neighbor_distance) * (mag > 0.01))[0]
+        neighbor_idx = np.where(incell * (atom_type == cen_type) * (mag < nearest_neighbor_distance) * (mag > 0.01))[0]
 
         # Sort order by distance to central atom
         srt_idx = np.argsort(mag[neighbor_idx])
@@ -459,22 +559,59 @@ class Properties:
         """Return latex table of structure properties from CIF"""
         return fc.cif2table(self.xtl.cif)
 
+    def beamline_info(self, energy_kev=None):
+        """Prints various properties useful for experiments"""
+
+        out = '\n'
+        out += '-----------%s-----------\n' % self.xtl.name
+        out += ' Formula: {}\n'.format(self.molname())
+        out += 'Magnetic: {}\n'.format(self.xtl.Structure.ismagnetic())
+        out += '  Weight: %5.2f g/mol\n' % (self.weight())
+        out += ' Density: %5.2f g/cm\n' % (self.density())
+        # Symmetry
+        out += '\nSymmetry: %r\n' % self.xtl.Symmetry
+        # Cell info
+        out += self.xtl.Cell.info()
+
+        if energy_kev is not None:
+            out += '\nEnergy = %s\n' % energy_kev
+            # Xray edges
+            out += '  Near X-Ray Edges:\n'
+            for edge_str, edge_en in zip(*self.xray_edges()):
+                if abs(edge_en - energy_kev) < 1:
+                    out += '    %5s : %7.4f keV\n' % (edge_str, edge_en)
+            # Absorption
+            out += '   Absorption coef: {} um^-1\n'.format(self.absorption(energy_kev))
+            out += 'Attenuation length: {} um\n'.format(self.xray_attenuation_length(energy_kev))
+            out += '  |Q| @ tth=180deg: {} A^-1\n'.format(fc.calqmag(180, energy_kev))
+            out += '    d @ tth=180deg: {} A\n'.format(fc.caldspace(180, energy_kev))
+            out += '           Max HKL: ({:d},{:d},{:d})\n'.format(*self.xtl.Cell.max_hkl(energy_kev, 180))
+            allhkl = self.xtl.Scatter.get_hkl(True, False, energy_kev=energy_kev)
+            out += '          No. Refs: {}\n'.format(len(allhkl))
+        else:
+            # Xray edges
+            out += '\nX-Ray Edges:\n'
+            for edge_str, edge_en in zip(*self.xray_edges()):
+                out += '    %2s : %7.4f keV\n' % (edge_str, edge_en)
+        out += '\n'
+        return out
+
     def info(self):
         """Prints various properties of the crystal"""
 
-        out = ''
-        out += '-----------%s-----------'%self.xtl.name
-        out += ' Weight: %5.2f g/mol' %(self.weight())
-        out += ' Volume: %5.2f A^3' %(self.volume())
-        out += 'Density: %5.2f g/cm' %(self.density())
-        out += '\nAtoms:'
+        out = '\n'
+        out += '-----------%s-----------\n'%self.xtl.name
+        out += ' Weight: %5.2f g/mol\n' %(self.weight())
+        out += ' Volume: %5.2f A^3\n' %(self.volume())
+        out += 'Density: %5.2f g/cm\n' %(self.density())
+        out += '\nAtoms:\n'
         types = np.unique(self.xtl.Structure.type)
         props = fc.atom_properties(types) # returns a numpy structured array
         prop_names = props.dtype.names
         for key in prop_names:
             ele = '%20s :' % key
             ele += ''.join([' %10s :' %(item) for item in props[key]])
-            out += ele
+            out += ele + '\n'
         return out
 
     def __repr__(self):
