@@ -10,8 +10,8 @@ By Dan Porter, PhD
 Diamond
 2019
 
-Version 2.2
-Last updated: 27/04/20
+Version 2.3.0
+Last updated: 26/01/21
 
 Version History:
 10/11/17 0.1    Program created
@@ -21,11 +21,14 @@ Version History:
 13/12/19 2.0    Changed to internal tkgui package
 19/04/20 2.1    Added multi-crystal, write-cif buttons, changed layout of scattering gui
 27/04/20 2.2    Added SelectionBox to basic_widgets and spacegroup entry to SymmetryGui
+15/10/20 2.2.1  Slight correction to SymmetryGui - no longer adds 'm' to labels
+21/01/21 2.2.2  Added 'xray dispersion' scattering option
+26/01/21 2.3.0  Refactored properites into new file, added x-ray interactions GUI
 
 @author: DGPorter
 """
 
 from .crystal import CrystalGui
 
-__version__ = '2.2'
-__date__ = '27/04/20'
+__version__ = '2.3.0'
+__date__ = '26/01/20'
